@@ -5,6 +5,7 @@ import imageio
 import yaml
 from threading import Lock
 
+MUJOCO_VERSION=tuple(map(int, mujoco.__version__.split('.')))
 
 class Callbacks:
     def __init__(self, hide_menus):
@@ -18,6 +19,7 @@ class Callbacks:
         self._last_mouse_x = 0
         self._last_mouse_y = 0
         self._paused = False
+        self._hide_graph = False
         self._transparent = False
         self._contacts = False
         self._joints = False
@@ -97,6 +99,9 @@ class Callbacks:
                 self.model.geom_rgba[:, 3] /= 5.0
             else:
                 self.model.geom_rgba[:, 3] *= 5.0
+        # Toggle Graph overlay
+        elif key == glfw.KEY_G:
+            self._hide_graph = not self._hide_graph
         # Display inertia
         elif key == glfw.KEY_I:
             self._inertias = not self._inertias
@@ -250,18 +255,34 @@ class Callbacks:
             rely = (self.viewport.height - y) / height
             selpnt = np.zeros((3, 1), dtype=np.float64)
             selgeom = np.zeros((1, 1), dtype=np.int32)
+            selflex = np.zeros((1, 1), dtype=np.int32)
             selskin = np.zeros((1, 1), dtype=np.int32)
-            selbody = mujoco.mjv_select(
-                self.model,
-                self.data,
-                self.vopt,
-                aspectratio,
-                relx,
-                rely,
-                self.scn,
-                selpnt,
-                selgeom,
-                selskin)
+
+            if MUJOCO_VERSION>=(3,0,0):
+                selbody = mujoco.mjv_select(
+                    self.model,
+                    self.data,
+                    self.vopt,
+                    aspectratio,
+                    relx,
+                    rely,
+                    self.scn,
+                    selpnt,
+                    selgeom,
+                    selflex,
+                    selskin)
+            else:
+                selbody = mujoco.mjv_select(
+                    self.model,
+                    self.data,
+                    self.vopt,
+                    aspectratio,
+                    relx,
+                    rely,
+                    self.scn,
+                    selpnt,
+                    selgeom,
+                    selskin)
 
             # set lookat point, start tracking is requested
             if selmode == 2 or selmode == 3:
