@@ -2,8 +2,11 @@ import mujoco
 import glfw
 import numpy as np
 import time
+import pathlib
+import yaml
 from .callbacks import Callbacks
 
+MUJOCO_VERSION=tuple(map(int, mujoco.__version__.split('.')))
 
 class MujocoViewer(Callbacks):
     def __init__(
@@ -27,6 +30,9 @@ class MujocoViewer(Callbacks):
         # keep true while running
         self.is_alive = True
 
+        self.CONFIG_PATH = pathlib.Path.joinpath(
+            pathlib.Path.home(), ".config/mujoco_viewer/config.yaml")
+
         # glfw init
         glfw.init()
 
@@ -65,9 +71,60 @@ class MujocoViewer(Callbacks):
         self.cam = mujoco.MjvCamera()
         self.scn = mujoco.MjvScene(self.model, maxgeom=10000)
         self.pert = mujoco.MjvPerturb()
+
         self.ctx = mujoco.MjrContext(
             self.model, mujoco.mjtFontScale.mjFONTSCALE_150.value)
 
+        width, height = glfw.get_framebuffer_size(self.window)
+
+        # figures for creating 2D plots
+        max_num_figs = 3
+        self.figs = []
+        width_adjustment = width % 4
+        fig_w, fig_h = int(width / 4), int(height / 4)
+        for idx in range(max_num_figs):
+            fig = mujoco.MjvFigure()
+            mujoco.mjv_defaultFigure(fig)
+            fig.flg_extend = 1
+            self.figs.append(fig)
+
+        # load camera from configuration (if available)
+        pathlib.Path(
+            self.CONFIG_PATH.parent).mkdir(
+            parents=True,
+            exist_ok=True)
+        pathlib.Path(self.CONFIG_PATH).touch(exist_ok=True)
+        with open(self.CONFIG_PATH, "r") as f:
+            try:
+                cam_config = {
+                    "type": self.cam.type,
+                    "fixedcamid": self.cam.fixedcamid,
+                    "trackbodyid": self.cam.trackbodyid,
+                    "lookat": self.cam.lookat.tolist(),
+                    "distance": self.cam.distance,
+                    "azimuth": self.cam.azimuth,
+                    "elevation": self.cam.elevation
+                }
+                load_config = yaml.safe_load(f)
+                if isinstance(load_config, dict):
+                    for key, val in load_config.items():
+                        if key in cam_config.keys():
+                            cam_config[key] = val
+                if cam_config["type"] == mujoco.mjtCamera.mjCAMERA_FIXED:
+                    if cam_config["fixedcamid"] < self.model.ncam:
+                        self.cam.type = cam_config["type"]
+                        self.cam.fixedcamid = cam_config["fixedcamid"]
+                if cam_config["type"] == mujoco.mjtCamera.mjCAMERA_TRACKING:
+                    if cam_config["trackbodyid"] < self.model.nbody:
+                        self.cam.type = cam_config["type"]
+                        self.cam.trackbodyid = cam_config["trackbodyid"]
+                self.cam.lookat = np.array(cam_config["lookat"])
+                self.cam.distance = cam_config["distance"]
+                self.cam.azimuth = cam_config["azimuth"]
+                self.cam.elevation = cam_config["elevation"]
+            except yaml.YAMLError as e:
+                print(e)
+
         # get viewport
         self.viewport = mujoco.MjrRect(
             0, 0, framebuffer_width, framebuffer_height)
@@ -76,6 +133,51 @@ class MujocoViewer(Callbacks):
         self._overlay = {}
         self._markers = []
 
+    def add_line_to_fig(self, line_name, fig_idx=0):
+        assert isinstance(line_name, str), \
+            "Line name must be a string."
+
+        fig = self.figs[fig_idx]
+        if line_name.encode('utf8') == b'':
+            raise Exception(
+                "Line name cannot be empty."
+            )
+        if line_name.encode('utf8') in fig.linename:
+            raise Exception(
+                "Line name already exists in this plot."
+            )
+
+        # this assumes all lines added by user have a non-empty name
+        linecount = fig.linename.tolist().index(b'')
+
+        # we want to add the line after the last non-empty index
+        fig.linename[linecount] = line_name
+
+        # assign x values
+        for i in range(mujoco.mjMAXLINEPNT):
+            fig.linedata[linecount][2*i] = -float(i)
+
+    def add_data_to_line(self, line_name, line_data, fig_idx=0):
+        fig = self.figs[fig_idx]
+
+        try:
+            _line_name = line_name.encode('utf8')
+            linenames = fig.linename.tolist()
+            line_idx = linenames.index(_line_name)
+        except ValueError:
+            raise Exception(
+                "line name is not valid, add it to list before calling update"
+            )
+
+        pnt = min(mujoco.mjMAXLINEPNT, fig.linepnt[line_idx] + 1)
+        # shift data
+        for i in range(pnt-1, 0, -1):
+            fig.linedata[line_idx][2*i + 1] = fig.linedata[line_idx][2*i - 1]
+
+        # assign new
+        fig.linepnt[line_idx] = pnt
+        fig.linedata[line_idx][1] = line_data
+
     def add_marker(self, **marker_params):
         self._markers.append(marker_params)
 
@@ -163,6 +265,10 @@ class MujocoViewer(Callbacks):
             topleft,
             "[J]oints",
             "On" if self._joints else "Off")
+        add_overlay(
+            topleft,
+            "[G]raph Viewer",
+            "Off" if self._hide_graph else "On")
         add_overlay(
             topleft,
             "[I]nertia",
@@ -196,6 +302,8 @@ class MujocoViewer(Callbacks):
                     topleft,
                     "Advance simulation by one step",
                     "[right arrow]")
+        add_overlay(topleft, "Toggle geomgroup visibility (0-5)",
+                    ",".join(["On" if g else "Off" for g in self.vopt.geomgroup]))
         add_overlay(
             topleft,
             "Referenc[e] frames",
@@ -206,15 +314,20 @@ class MujocoViewer(Callbacks):
             add_overlay(topleft, "Cap[t]ure frame", "Saved as %s" % fname)
         else:
             add_overlay(topleft, "Cap[t]ure frame", "")
-        add_overlay(topleft, "Toggle geomgroup visibility (0-5)",
-                    ",".join(["On" if g else "Off" for g in self.vopt.geomgroup]))
 
         add_overlay(
             bottomleft, "FPS", "%d%s" %
             (1 / self._time_per_render, ""))
-        add_overlay(
-            bottomleft, "Solver iterations", str(
-                self.data.solver_iter + 1))
+
+        if MUJOCO_VERSION>=(3,0,0):
+            add_overlay(
+                bottomleft, "Max solver iters", str(
+                    max(self.data.solver_niter) + 1))
+        else:
+            add_overlay(
+                bottomleft, "Solver iterations", str(
+                    self.data.solver_iter + 1))
+
         add_overlay(
             bottomleft, "Step", str(
                 round(
@@ -226,7 +339,7 @@ class MujocoViewer(Callbacks):
         mujoco.mjv_applyPerturbPose(self.model, self.data, self.pert, 0)
         mujoco.mjv_applyPerturbForce(self.model, self.data, self.pert)
 
-    def read_pixels(self, camid=None):
+    def read_pixels(self, camid=None, depth=False):
         if self.render_mode == 'window':
             raise NotImplementedError(
                 "Use 'render()' in 'window' mode.")
@@ -251,13 +364,17 @@ class MujocoViewer(Callbacks):
             self.scn)
         # render
         mujoco.mjr_render(self.viewport, self.scn, self.ctx)
+        shape = glfw.get_framebuffer_size(self.window)
 
-        img = np.zeros(
-            (glfw.get_framebuffer_size(
-                self.window)[1], glfw.get_framebuffer_size(
-                self.window)[0], 3), dtype=np.uint8)
-        mujoco.mjr_readPixels(img, None, self.viewport, self.ctx)
-        return np.flipud(img)
+        if depth:
+            rgb_img = np.zeros((shape[1], shape[0], 3), dtype=np.uint8)
+            depth_img = np.zeros((shape[1], shape[0], 1), dtype=np.float32)
+            mujoco.mjr_readPixels(rgb_img, depth_img, self.viewport, self.ctx)
+            return (np.flipud(rgb_img), np.flipud(depth_img))
+        else:
+            img = np.zeros((shape[1], shape[0], 3), dtype=np.uint8)
+            mujoco.mjr_readPixels(img, None, self.viewport, self.ctx)
+            return np.flipud(img)
 
     def render(self):
         if self.render_mode == 'offscreen':
@@ -276,8 +393,10 @@ class MujocoViewer(Callbacks):
             self._create_overlay()
 
             render_start = time.time()
-            self.viewport.width, self.viewport.height = glfw.get_framebuffer_size(
-                self.window)
+
+            width, height = glfw.get_framebuffer_size(self.window)
+            self.viewport.width, self.viewport.height = width, height
+
             with self._gui_lock:
                 # update scene
                 mujoco.mjv_updateScene(
@@ -307,6 +426,20 @@ class MujocoViewer(Callbacks):
                         t1,
                         t2,
                         self.ctx)
+
+                # handle figures
+                if not self._hide_graph:
+                    for idx, fig in enumerate(self.figs):
+                        width_adjustment = width % 4
+                        x = int(3 * width / 4) + width_adjustment
+                        y = idx * int(height / 4)
+                        viewport = mujoco.MjrRect(
+                            x, y, int(width / 4), int(height / 4))
+
+                        has_lines = len([i for i in fig.linename if i != b''])
+                        if has_lines:
+                            mujoco.mjr_figure(viewport, fig, self.ctx)
+
                 glfw.swap_buffers(self.window)
             glfw.poll_events()
             self._time_per_render = 0.9 * self._time_per_render + \
